/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.sample.dataservice;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.client.Options;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.util.XMLPrettyPrinter;
import org.apache.neethi.Policy;
import org.apache.neethi.PolicyEngine;
import org.apache.rampart.RampartMessageData;
import org.apache.rampart.policy.model.CryptoConfig;
import org.apache.rampart.policy.model.RampartConfig;
import org.apache.ws.security.WSPasswordCallback;

public class SecuredDataServiceClient implements CallbackHandler {

    private static EndpointReference targetEPR = new EndpointReference(
            "http://localhost:9763/services/SecuredDSSample");
    
    

    public static void main(String args[]) {
        try {

            if (args.length < 2) {
                System.out.println("Please enter policyXml file");
                return;
            }
            String policyXml = args[0].trim();
            String carbonHome = args[1];

            loadSSLProperties(carbonHome);

            if(policyXml.endsWith("scenario1-policy.xml")){
                targetEPR = new EndpointReference("https://localhost:9443/services/SecuredDSSample");
            }
            
            Options options1 = new Options();
            options1.setTo(targetEPR);
            options1.setAction("urn:customersInBoston");

            ServiceClient client = new ServiceClient();
            client.engageModule("rampart");
            Policy policy = loadPolicy(policyXml, carbonHome);
            
            options1.setProperty(RampartMessageData.KEY_RAMPART_POLICY, policy);
            client.setOptions(options1);

            OMElement payload1 = getCustomerContactDetailsPayload();
            OMElement result1 = client.sendReceive(payload1);
            System.out.println("\n\nRequesting contact details of customers in Boston...\n");

            XMLPrettyPrinter.prettify(result1, System.out);

        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }
    
    private static Policy loadPolicy(String xmlPath, String carbonHome) throws Exception {
        StAXOMBuilder builder = new StAXOMBuilder(xmlPath);
        Policy policy = PolicyEngine.getPolicy(builder.getDocumentElement());

        RampartConfig rc = new RampartConfig();

        rc.setUser("client");
        rc.setEncryptionUser("service");
        rc.setPwCbClass(SecuredDataServiceClient.class.getName());

        CryptoConfig sigCryptoConfig = new CryptoConfig();

        sigCryptoConfig.setProvider("org.apache.ws.security.components.crypto.Merlin");

        File file = new File("security" + File.separator + "keys" + File.separator + "client.jks");

        if (!file.exists()) {
            throw new Exception("Key store cannot be found");
        }

        String filepath = file.getCanonicalPath();

        Properties prop1 = new Properties();
        prop1.put("org.apache.ws.security.crypto.merlin.keystore.type", "JKS");
        prop1.put("org.apache.ws.security.crypto.merlin.file", filepath);
        prop1.put("org.apache.ws.security.crypto.merlin.keystore.password", "apache");
        sigCryptoConfig.setProp(prop1);

        CryptoConfig encrCryptoConfig = new CryptoConfig();
        encrCryptoConfig.setProvider("org.apache.ws.security.components.crypto.Merlin");

        Properties prop2 = new Properties();

        prop2.put("org.apache.ws.security.crypto.merlin.keystore.type", "JKS");
        prop2.put("org.apache.ws.security.crypto.merlin.file", filepath);
        prop2.put("org.apache.ws.security.crypto.merlin.keystore.password", "apache");
        encrCryptoConfig.setProp(prop2);

        rc.setSigCryptoConfig(sigCryptoConfig);
        rc.setEncrCryptoConfig(encrCryptoConfig);

        policy.addAssertion(rc);

        return policy;
    }

    private static void loadSSLProperties(String carbonHome) throws Exception {

        String keystore = carbonHome + File.separator + "conf" + File.separator + "wso2carbon.jks";

        File file = new File(keystore);
        if (!file.exists()) {
            throw new Exception("SSL key store cannot be found");
        }

        String filepath = file.getCanonicalPath();

        System.getProperties().remove("javax.net.ssl.trustStore");
        System.getProperties().remove("javax.net.ssl.trustStoreType");
        System.getProperties().remove("javax.net.ssl.trustStorePassword");

        System.setProperty("javax.net.ssl.trustStore", filepath);
        System.setProperty("javax.net.ssl.trustStoreType", "JKS");
        System.setProperty("javax.net.ssl.trustStorePassword", "wso2carbon");

    }

    private static OMElement getCustomerContactDetailsPayload() {
        OMFactory fac = OMAbstractFactory.getOMFactory();
        OMNamespace omNs = fac.createOMNamespace("http://example1.org/example1", "example1");
        OMElement method = fac.createOMElement("contactDetailsOfCustomersInBoston", omNs);
        return method;
    }

    public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
        WSPasswordCallback cb = (WSPasswordCallback) callbacks[0];
        cb.setPassword("apache");
    }

}
